﻿using CommandLine;
using CommandLine.Text;
using System;
using System.Linq;
using System.Messaging;

/*
 *         Progetto: Progetto "Pagamenti MSMQ"
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 24/01/2016
 *  
 *      Descrizione: Esegue test di invio per verificare il funzionamento delle code MSMQ.
 *                   Il programma accetta un argomento contente il numero di messaggi da inviare. 
 *                   
 *    Esempio d'uso: TestScritturaCoda.exe -d testlabsis.fauser.edu -r 10.0.2.1 -c CodaAdmin -n 1
 *                   
 */

namespace PagamentiMSMQ
{
    class Program
    {
        class Opzioni
        {
            [Option('d', "dest", Required = false, DefaultValue = ".", HelpText = "Nome o indirizzo IP dell'host a cui inviare il messaggio.")]
            public string HostDest { get; set; }

            [Option('r', "risp", Required = false, DefaultValue = ".", HelpText = "Nome o indirizzo IP dell'host che riceve i riscontri e la risposta.")]
            public string HostRisp { get; set; }

            [Option('c', "codarisp", Required = false, DefaultValue = "Cancella", HelpText = "Nome della coda dei riscontri e delle risposte.")]
            public string CodaRisp { get; set; }

            [Option('n', "num", Required = false, DefaultValue = 1, HelpText = "Numero di messaggi da inviare.")]
            public int NumMessaggi { get; set; }

            [HelpOption(HelpText = "Mostra questo messaggio.")]
            public string GetUsage()
            {
                return HelpText.AutoBuild(this,
                  (HelpText current) => HelpText.DefaultParsingErrorsHandler(this, current));
            }
        }

        const string PatternCodaLocale = @"{0}\Private$\{1}";
        const string PatternCodaRemota = @"FormatName:DIRECT=TCP:{0}\Private$\{1}";

        static void Main(string[] args)
        {
            var opzioni = new Opzioni();
            if (!CommandLine.Parser.Default.ParseArguments(args, opzioni))
            {
                opzioni.GetUsage();
                return;
            }

            if (opzioni.HostDest != null)
                Console.WriteLine(opzioni.HostDest);

            try
            {
                //MessageQueue coda = new MessageQueue(@"FormatName:DIRECT=TCP:192.168.0.35\Private$\Pagamenti");

                MessageQueue coda = new MessageQueue(String.Format(opzioni.HostDest.Equals(".") ? PatternCodaLocale : PatternCodaRemota,
                                                                    opzioni.HostDest, "Pagamenti"));

                //MessageQueue codaAdmin = new MessageQueue(@"FormatName:DIRECT=TCP:10.0.49.62\Private$\cancella");
                MessageQueue codaAdmin = new MessageQueue(String.Format(opzioni.HostDest.Equals(".") ? PatternCodaLocale : PatternCodaRemota,
                                                    opzioni.HostRisp, opzioni.CodaRisp));



                codaAdmin.MessageReadPropertyFilter.SetAll();
                ((XmlMessageFormatter)codaAdmin.Formatter).TargetTypeNames = new string[] { "System.String,mscorlib" };


                var r = new Random();


                for (int i = 0; i < opzioni.NumMessaggi; i++)
                {
                    Message m = new Message();
                    InfoPagamento inf = InfoPagamento.Test(r);
                    
                    m.Body = inf;
 
                    m.ResponseQueue = codaAdmin;
                    m.AdministrationQueue = codaAdmin;
                    m.AcknowledgeType = AcknowledgeTypes.FullReceive;

                    coda.Send(m);
                    Console.WriteLine(String.Format("Inviato messaggio - id {0}", m.Id));
                }

                bool finito = false;
                

                while (!finito)
                {
                    try
                    {
                        Message msg = codaAdmin.Receive(TimeSpan.FromSeconds(1));

                        if (msg.MessageType == MessageType.Acknowledgment)
                        {
                            Console.WriteLine("ACK del messaggio {0}", msg.CorrelationId);
                        }
                        else if (msg.MessageType == MessageType.Normal)
                        {
                            Console.WriteLine("Risposta al messaggio {0}: {1}", msg.CorrelationId, msg.Body.ToString());
                        }
                    }
                    catch (MessageQueueException)
                    {
                        finito = true;
                    }

                }


            }
            catch (Exception e)
            {
                Console.Error.WriteLine("Errore: {0}", e.Message);
            }

        }
    }
}
