﻿using NLog;
using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlServerCe;
using System.IO;
using System.Timers;

/*
 *         Progetto: Progetto "Pagamenti MSMQ"
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 24/01/2016
 *  
 *      Descrizione: Routine di gestione del database SQL contenente i dettagli dei pagamenti.
 *                   Il motore di database è di tipo "Microsoft SQL Server CE".
 *                   
 */

namespace PagamentiMSMQ
{
    public sealed class UtilityDB
    {
        static readonly UtilityDB _instance = new UtilityDB();
        public static UtilityDB Instance
        {
            get
            {
                return _instance;
            }
        }

        private Logger logger = LogManager.GetCurrentClassLogger();
        private SqlCeConnection conn = null;
        private SqlCeCommand cmdScrittura = null;
        private SqlCeCommand cmdLettura = null;
        private Object mutex = new Object();
        private bool flagDbModificato = true;
        private Object mutexModifica = new Object();
        private Timer timerNotifica = new Timer(1000);
        
        public event EventHandler notificaNuovaRevisione;


        public bool DbModificato
        {
            get
            {
                lock (mutexModifica)
                {
                    return flagDbModificato;
                }
            }
            set
            {
                lock (mutexModifica)
                {
                    flagDbModificato = value;
                }
            }
        }

        private void NuovaRevisione()
        {
            lock (mutexModifica)
            {
                flagDbModificato = true;
            }
        }

        public string NomeFile
        {
            get
            {
                // Salva in c:\Windows\ServiceProfiles\LocalService\AppData\Local

                string destDir = Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData);
                string s = destDir + @"\Pagamenti.sdf";
                return s;
            }
        }

        string ConnStr
        {
            get
            {
                string s = String.Format(@"Data Source={0}; Case Sensitive=True;", NomeFile);
                return s;
            }
        }

        UtilityDB()
        {
            timerNotifica.Elapsed += EseguiNotifica;
            timerNotifica.Enabled = true;

            lock (mutex)
            {
                Crea(false);

                conn = new SqlCeConnection(ConnStr);
                cmdScrittura = new SqlCeCommand("INSERT INTO Pagamenti(NomeVenditore, IdVenditore, NomeAcquirente, NumeroCarta, TipoCarta, Importo) " +
                                                "VALUES (@NomeVenditore, @IdVenditore, @NomeAcquirente, @NumeroCarta, @TipoCarta, @Importo)", conn);
                cmdScrittura.Parameters.Add(new SqlCeParameter("@NomeVenditore", SqlDbType.NVarChar, 50));
                cmdScrittura.Parameters.Add(new SqlCeParameter("@IdVenditore", SqlDbType.Int));
                cmdScrittura.Parameters.Add(new SqlCeParameter("@NomeAcquirente", SqlDbType.NVarChar, 50));
                cmdScrittura.Parameters.Add(new SqlCeParameter("@NumeroCarta", SqlDbType.NVarChar, 30));
                cmdScrittura.Parameters.Add(new SqlCeParameter("@TipoCarta", SqlDbType.SmallInt));
                var paramImportoScrittura = new SqlCeParameter("@Importo", SqlDbType.Decimal);
                paramImportoScrittura.Precision = 9;
                paramImportoScrittura.Scale = 2;
                cmdScrittura.Parameters.Add(paramImportoScrittura);

                cmdLettura = new SqlCeCommand("SELECT TOP 20 NomeVenditore, IdVenditore, NomeAcquirente, NumeroCarta, TipoCarta, Importo, ID, Data FROM Pagamenti " +
                                                "ORDER BY ID DESC", conn);
            }
        }

        public void Crea(bool eliminaSePresente = false)
        {
            try
            {
                logger.Trace("Creazione DB in corso...");

                if (File.Exists(NomeFile))
                    if (!eliminaSePresente)
                        return;
                    else
                        File.Delete(NomeFile);

                SqlCeEngine engine = new SqlCeEngine(ConnStr);
                engine.CreateDatabase();

                using (SqlCeConnection conn = new SqlCeConnection(ConnStr))
                {
                    try
                    {
                        conn.Open();
                        using (SqlCeCommand cmd = new SqlCeCommand(
                            @"CREATE TABLE Pagamenti (ID INT NOT NULL IDENTITY (1, 1), Data DATETIME DEFAULT GETDATE(), " +
                            @"NomeVenditore NVARCHAR(50) NOT NULL, IdVenditore INT NOT NULL, " +
                            @"NomeAcquirente NVARCHAR(50) NOT NULL, " + 
                            @"NumeroCarta NVARCHAR(30) NOT NULL, TipoCarta SMALLINT NOT NULL, " +
                            @"Importo NUMERIC(9, 2) NOT NULL, " +
                            @"CONSTRAINT pk_Pagamenti PRIMARY KEY (ID))"
                            , conn))
                        {
                            cmd.ExecuteNonQuery();
                        }
                    }
                    finally
                    {
                        conn.Close();
                    }
                }
            }
            catch (Exception)
            {
            }
        }

        public bool ScriviPagamento(InfoPagamento inf)
        {
            bool risultato = false;

            lock (mutex)
            {
                try
                {
                    if (conn.State != ConnectionState.Open) 
                        conn.Open();

                    cmdScrittura.Parameters["@NomeVenditore"].Value = inf.NomeVenditore;
                    cmdScrittura.Parameters["@IdVenditore"].Value = inf.IdVenditore;
                    cmdScrittura.Parameters["@NomeAcquirente"].Value = inf.NomeAcquirente;
                    cmdScrittura.Parameters["@NumeroCarta"].Value = inf.NumeroCarta;
                    cmdScrittura.Parameters["@TipoCarta"].Value = inf.TipoCarta;
                    cmdScrittura.Parameters["@Importo"].Value = inf.Importo;

                    try
                    {
                        cmdScrittura.ExecuteNonQuery();
                        logger.Trace("Record salvato nel database SQL.");
                        risultato = true;
                    }
                    catch (Exception e)
                    {
                        logger.Error(e);
                    }

                    NuovaRevisione();

                }
                catch (Exception e) { logger.Error(e); }

            }

            return risultato;
        }

        public InfoPagamentoEsteso[] LeggiPagamentiRecenti()
        {
            List<InfoPagamentoEsteso> risultato = new List<InfoPagamentoEsteso>();

            lock (mutex)
            {
                if (conn.State != ConnectionState.Open)
                    conn.Open();

                var r = cmdLettura.ExecuteReader();
                try
                {
                    while (r.Read())
                    {

                        InfoPagamentoEsteso inf = new InfoPagamentoEsteso();
                        inf.NomeVenditore = r.GetString(0);
                        inf.IdVenditore = r.GetInt32(1);
                        inf.NomeAcquirente = r.GetString(2);
                        inf.NumeroCarta = r.GetString(3);
                        inf.TipoCarta = r.GetInt16(4);
                        inf.Importo = (double) r.GetDecimal(5);
                        inf.Id = r.GetInt32(6);
                        inf.Data = r.GetDateTime(7).ToString("dd/MM/yyyy HH:mm:ss");
                        inf.ImportoValuta = String.Format("{0:N2} EUR", inf.Importo);
                        risultato.Add(inf);
                    }
                }
                finally
                {
                    r.Close();
                }
            }

            return risultato.ToArray();
        }
        void EseguiNotifica(object sender, ElapsedEventArgs e)
        {
            if (DbModificato)
            {
                notificaNuovaRevisione(this, EventArgs.Empty);
                DbModificato = false;
            }
            
        }
    }
}
